const API_BASE_URL = '/api/feed';


const postFeed = document.getElementById('postFeed');
const postTemplate = document.getElementById('post-template');
const addNewPost = document.getElementById('add-post');
const closeModalBtn = document.getElementById('closeModalBtn');
const modal = document.getElementById('create-new-modal');


addNewPost.addEventListener('click',  () => {
    modal.style.display = 'flex'
})

closeModalBtn.addEventListener('click', () => {
    modal.style.display = 'none' +
        ''
})

// maintaining state
let posts = [];

// getting all posts using api
async function fetchPosts() {
    try {
        const response = await fetch(API_BASE_URL);
        if (!response.ok) throw new Error('Failed to fetch posts');
        posts = await response.json();
        renderPosts();
    } catch (error) {
        console.error('Error fetching posts:', error);
    }
}



// render all posts after getting them
function renderPosts() {
    postFeed.innerHTML = ''; // clear any posts
    posts.forEach(post => renderPost(post)); // go over each item in post array and call the function renderPost on it
}

// a single post
async function renderPost(post) {
    const postElement = postTemplate.content.cloneNode(true);

    const deleteButton = postElement.querySelector('.delete-post');
    const editButton = postElement.querySelector('.edit-post');
    deleteButton.style.display = post.isDeletable ? 'block' : 'none';
    editButton.style.display = post.isEditable ? 'block' : 'none';
    postElement.querySelector('.author').textContent = post.authorName;
    postElement.querySelector('.author-title').textContent = post.authorOrganization;
    postElement.querySelector('.post-title').textContent = post.postTitle;
    postElement.querySelector('.post-description').textContent = post.postDescription;

    const postImage = postElement.querySelector('.post-image img');
    if (post.postImageUrl) {
        postImage.src = post.postImageUrl;
    } else {
        postImage.style.display = 'none';
    }

    // render the tags
    const tagsContainer = postElement.querySelector('.post-tags');
    if (post.tags && post.tags.length > 0) {
        post.tags.forEach(tag => {
            const tagSpan = document.createElement('span');
            tagSpan.innerHTML = `<i class="bi bi-tag"></i> ${tag}`;
            tagsContainer.appendChild(tagSpan);
        });
    }

    // set the count of the likes
    const likeCount = postElement.querySelector('.like-count');
    likeCount.textContent = post.likesCount || 0;

    // timestamp
    const timestamp = postElement.querySelector('.timestamp');
    timestamp.textContent = new Date(post.postTime).toLocaleDateString();

    // add a like when presses
    const likeButton = postElement.querySelector('.like-button');
    likeButton.addEventListener('click', () => handleLike(post.postId, likeCount));

    // Set data attributes for reference
    const postDiv = postElement.querySelector('.post');
    postDiv.dataset.postId = post.postId;

    //deleting post
    if (post.isDeletable) {
        deleteButton.addEventListener('click', async () => {
            if (confirm('Are you sure you want to delete this post?')) {
                try {
                    const response = await fetch(`${API_BASE_URL}/${post.postId}`, {
                        method: 'DELETE',
                        credentials: 'include'
                    });

                    if (!response.ok) {
                        throw new Error('Failed to delete post');
                    }

                    const postDiv = deleteButton.closest('.post');
                    postDiv.remove();
                    posts = posts.filter(p => p.postId !== post.postId);
                } catch (error) {
                    console.error('Error deleting post:', error);
                    alert('Error deleting post. Please try again.');
                }
            }
        });
    }

    postFeed.appendChild(postElement);
}

// add a like if user had not already liked and remove a like if already liked
async function handleLike(postId, likeCountElement) {
    try {
        // check if user had already liked
        const checkResponse = await fetch(`${API_BASE_URL}/${postId}/hasLiked?userId=${MOCK_USER_ID}`);
        const hasLiked = await checkResponse.json();

        const method = hasLiked ? 'DELETE' : 'POST';
        const response = await fetch(`${API_BASE_URL}/${postId}/like?userId=${MOCK_USER_ID}`, {
            method: method
        });

        if (!response.ok) throw new Error('Failed to update like');

        // update like count appropriately
        const currentCount = parseInt(likeCountElement.textContent);
        likeCountElement.textContent = hasLiked ? currentCount - 1 : currentCount + 1;

        // change color to show they have liked already
        const button = likeCountElement.closest('.like-button');
        button.classList.toggle('liked', !hasLiked);

    } catch (error) {
        console.error('Error updating like:', error);
    }
}

//* ************************* Submitting the form *********************** *//
const postForm = document.getElementById('post-form');


postForm.addEventListener('submit', (event) => {
    event.preventDefault();

    // getting form data,
    const postTitle = document.getElementById('postTitle').value;
    const postDescription = document.getElementById('postDescription').value;
    const postTagsInput = document.getElementById('postTags').value;

    // convert comma-separated tags to array and trim whitespace
    const tags = postTagsInput
        .split(',')
        .map(tag => tag.trim())
        .filter(tag => tag.length > 0); // removing empty tags

    const data = {
        postTitle,
        postDescription,
        tags // sending as array
    };

    console.log('Submitting post:', data);

    // Send post request
    fetch(`${API_BASE_URL}/add`, {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json',
        },
        credentials: 'include',
        body: JSON.stringify(data)
    })
        .then(response => {
            if (!response.ok) {
                throw new Error('Network response was not ok');
            }
            return response.text();
        })
        .then(result => {
            alert('Post added successfully');
            modal.style.display = 'none';
            fetchPosts();
        })
        .catch(error => {
            console.error('Error adding post:', error);
            alert('Error adding post. Please try again.');
        });
});
document.addEventListener('DOMContentLoaded', () => {
    fetchPosts();
});

// refresh posts each minute , though cannot show at moment since not online
// setInterval(fetchPosts, 60000);